# Source Code for 'Distributionally-Aware Kernelized Bandit Problems for Risk Aversion'
## Dependencies
This program is implemented in Python using the following standard packages:
- scipy (version 1.8.0)
- scikit-learn (version 1.0.2)
- numpy (version 1.21.5)

The author tested this program using Python 3.9.10.

## Modules/Scripts
The following modules/scripts are included in the 'ck_bandit' directory.
- base.py: a module for base classes of policies and environments.
- pols/gp_bandits.py: a module that provides implementation of proposed methods (CVPKE-UCB, MVPKE-UCB) and baselines (IGP-UCB, RAHBO).
- envs/synth_envs.py: a module for synthetic environments.
- run_policy.py: a module for running a policy in an environment.
- tests/run_pol_cvar_test.py: a script (or a unittest) for running the experiments for CVaR optimization.
- tests/run_pol_mv_test.py: a script for running the experiments for MV optimization.
- load_res_cvar.py: a module for loading saved experimental results in the case of CVaR optimization.
- load_res_mv.py: a module for loading saved experimental results in the case of MV optimization.

## How to reproduce the experiments in the paper
To reproduce the experiments for CVaR, run the following command:

    $ chmod u+x run_cvar_exp.sh && ./run_cvar_exp.sh

To reproduce the experiments for MV, run the following command:

    $ chmod u+x run_mv_exp.sh && ./run_mv_exp.sh

These commands save experimental results in the `results` directory.
One can load the saved results by using `cum_regret` function 
in `load_res_cvar` module (resp. `load_res_mv`)
in the case of cumulative CVaR (resp. MV) regret minimization.
Please refer to documentation of these functions for details.

## LICENSE
This program is licensed under BSD 3-Clause Clear License.
See LICENSE.md for details.

## Author
Sho Takemori, Fujitsu Limited, takemori.sho@fujitsu.com
