from abc import ABCMeta, abstractmethod
import numpy as np
from typing import Optional, Tuple


class BanditPolicy(metaclass=ABCMeta):
    @abstractmethod
    def select(self, arms: np.ndarray, rng: np.random.Generator) -> np.ndarray:
        pass

    @abstractmethod
    def update(self, selected_arm: np.ndarray, reward: float):
        pass

    @abstractmethod
    def name(self) -> str:
        pass

    def cvar_hat(self, arm: np.ndarray, alpha: float) -> Optional[float]:
        return

    def mv_hat(self, arm: np.ndarray, c: float) -> Optional[float]:
        return


class Environment(metaclass=ABCMeta):
    @abstractmethod
    def observation(self, arm: np.ndarray, rng: np.random.Generator) -> float:
        pass

    @abstractmethod
    def arms(self) -> np.ndarray:
        pass

    def best_arm_and_reward(self, alpha: float) -> Tuple[np.ndarray, float]:
        raise NotImplementedError()

    @abstractmethod
    def name(self) -> str:
        pass

    def min_max_of_q(self, p: float) -> Tuple[float, float]:
        raise NotImplementedError()

    @abstractmethod
    def cvar(self, arm: np.ndarray, alpha: float) -> float:
        pass

    # methods for MV

    @abstractmethod
    def best_mv_arm_and_reward(self, c: float) -> Tuple[np.ndarray, float]:
        pass

    @abstractmethod
    def mean_var(self, arm: np.ndarray, c: float) -> float:
        pass
