import pickle
import ck_bandit.run_policy as ck_run_pol
import ck_bandit.pols.gp_bandits as gp_bandits
import ck_bandit.pols.pols_misc as ck_pols_misc
from multiprocessing import Pool
from typing import Dict

from ck_bandit.constants import *
import numpy as np


def load_best_params():
    with open(BEST_PARAMS_MV_PATH, 'rb') as fp:
        return pickle.load(fp)


def cumreg_mvs_using_env_wrapper(dim: int, env_type: str, pol, nrounds, env_seed, mv_c, is_eval=True) -> np.ndarray:
    if is_eval:
        obs_seed = EVAL_OBS_SEED
        env = ck_run_pol.ENV_FNS_EVAL[env_type](dim=dim, seed=env_seed)
    else:
        obs_seed = PARAM_SEARCH_OBS_SEED
        env = ck_run_pol.ENV_FNS_PARAM_SEARCH[env_type](dim=dim, seed=env_seed)
    return ck_run_pol.cumreg_mvs_using_env(pol, env, nrounds, obs_seed, mv_c, norm_fac=1.0)


def cum_regret(env_type: str) -> Dict[str, np.ndarray]:
    '''
    Parameters
    ----------
    env_type: str should be 'normal' or 'log_normal' 
    ----------

    Returns
    ----------
    Dict[str, np.ndarray]

    returns a dictionay whose set of keys is {'RAHBO', 'MVPKE-UCB'}
    and the corresponding value is a numpy array of shape (10, 1000).
    The (i, j) entry of the corresponding array represents 
    normalized cummulative MV regret up to round j 
    when the policy interatcts with the i-th synthetic environment.
    '''
    mv_c = 1.0
    dim = DIMS[0]
    is_eval = True
    nrounds = NROUNDS_EVAL

    d = load_best_params()
    prm_rahbo = d['rahbo'][(dim, env_type, mv_c)]
    prm_mvucb = d['mv_ucb'][(dim, env_type, mv_c)]

    pols = [gp_bandits.RAHBO(prm_rahbo, mv_c),
            gp_bandits.MVKMEUCB(prm_mvucb, mv_c), ]

    pol_names = ['RAHBO', 'MVPKE-UCB']

    env_seeds = EVAL_SEEDS

    res = {}
    norm_facs = []
    for env_seed in env_seeds:
        with Pool(1) as pl:
            rand_pol = ck_pols_misc.RandomPolicy()
            args = (dim, env_type, rand_pol, nrounds, env_seed, mv_c, is_eval)
            cum_reg_rand = pl.apply(cumreg_mvs_using_env_wrapper, args)
            norm_factor = cum_reg_rand[nrounds - 1]
            cum_reg_rand /= norm_factor
        norm_facs.append(norm_factor)

    for pol, pol_name in zip(pols, pol_names):
        cum_regs = []
        for env_seed, norm_fac in zip(env_seeds, norm_facs):
            with Pool(1) as pl:
                args = (dim, env_type, pol, nrounds, env_seed, mv_c, is_eval)
                cum_reg = pl.apply(cumreg_mvs_using_env_wrapper, args)
                cum_reg /= norm_fac
            cum_regs.append(cum_reg)
        cum_regs = np.array(cum_regs)
        res[pol_name] = cum_regs
    return res
